@@ -18,13 +18,13 @@ from odf.text import P,  H,  Span, ListStyle,  ListLevelStyleBullet,  List,  Lis
 from odf.table import Table, TableColumn, TableRow, TableCell,  TableHeaderRows
 from odf.draw import Frame, Image
 from odf.dc import Creator, Description, Title, Date, Subject
-from odf.meta import InitialCreator
+from odf.meta import InitialCreator, Keyword, CreationDate
 import odf.element
 
 from odf.config import ConfigItem, ConfigItemMapEntry, ConfigItemMapIndexed, ConfigItemMapNamed,  ConfigItemSet
 from odf.office import Annotation
 
-from officegenerator.commons import makedirs,  number2column,  number2row,  Coord, Range,  Percentage, Currency,  __version__
+from officegenerator.commons import makedirs,  number2column,  number2row,  Coord, Range,  Percentage, Currency
 
 try:
     t=gettext.translation('officegenerator',pkg_resources.resource_filename("officegenerator","locale"))
@@ -164,15 +164,25 @@ class ODF:
         self.filename=filename
         self.images={}
         
-                
-    def setMetadata(self, title,  subject, creator):
+    ## Set metadata in odf document
+    ## @param title String with the title of the document
+    ## @param subject String with a brief of the document
+    ## @param creator String with the author of the document
+    ## @param description String with a larga description of the document
+    ## @param keywords String with keywords separated by space
+    ## @param creationdate Naive datetime with the creation date and time
+    def setMetadata(self, title="",  subject="", creator="", description="", keywords="", creationdate=datetime.datetime.now()):
         for e in self.doc.meta.childNodes:
             self.doc.meta.removeChild(e)
-        self.doc.meta.addElement(Description(text=_("This document has been generated with OfficeGenerator v{}".format(__version__))))
+        self.doc.meta.addElement(Description(text=description))
         self.doc.meta.addElement(Title(text=title))
         self.doc.meta.addElement(Subject(text=subject))
         self.doc.meta.addElement(Creator(text=creator))
         self.doc.meta.addElement(InitialCreator(text=creator))
+        self.doc.meta.addElement(Keyword(text=keywords))
+        d=Date()
+        d.addText(creationdate.strftime("%Y-%m-%dT%H:%M:%S"))
+        self.doc.meta.addElement(CreationDate(text=d))
 
     ## Adds an image to self.images dictionary. We add to a dictionary in order to reuse the same image in a directory
     ##
@@ -375,34 +385,36 @@ class ODT(ODF):
         print ("tag {} not found".format(tag))
         return None, None
 
-        
-    ##
+
+    ## Converts saved odt to pdf. It will have the same file name but with .pdf extension
     def convert_to_pdf(self):
-        os.system("lowriter --headless --convert-to pdf {}".format(self.filename))
-        
+        os.system("lowriter --headless --convert-to pdf '{}'".format(self.filename))
+
     def __setCursor(self, e):
         self.cursor=e
         self.cursorParent=e.parentNode
         if self.cursorParent==None:
             print("Parent of '{}' is None".format(self.cursor))
 
+    ## Check if self.filename is different to self.template, create directory and saves the file
     def save(self):
         if  self.filename==self.template:
             print(_("You can't overwrite a readed odt"))
-            return        
+            return
+        makedirs(os.path.dirname(self.filename))
         self.doc.save( self.filename)
 
-
+    ## Adds an empty paragraph
+    ## @param style String with the style name
+    ## @param number Integer with the number of times to repeat the empty paragraph
+    ## @param after True: insert after self.cursor element. False: insert before self.cursor element. None: Just return element
     def emptyParagraph(self, style="Standard", number=1, after=True):
         for i in range(number):
             self.simpleParagraph("",style, after)
 
-
-
-
-
     ## Creates the document title
     ## @param text String with the title
+    ## @param after True: insert after self.cursor element. False: insert before self.cursor element. None: Just return element
     def title(self, text, after=True):
         p=P(stylename="Title", text=text)
         return self.insertInCursor(p, after)
@@ -410,6 +422,7 @@ class ODT(ODF):
 
     ## Creates the document title
     ## @param text String with the title
+    ## @param after True: insert after self.cursor element. False: insert before self.cursor element. None: Just return element
     def subtitle(self, text, after=True):
         p=P(stylename="Subtitle", text=text)
         return self.insertInCursor(p, after)
@@ -422,6 +435,7 @@ class ODT(ODF):
     ## @param sizes Integer list with sizes in cm
     ## @param fontsize Integer in pt
     ## @param name str or None. Sets the object name. Appears in LibreOffice navigator. If none table will be named to "Table.Sequence"
+    ## @param after True: insert after self.cursor element. False: insert before self.cursor element. None: Just return element
     def table(self, header, data, sizes, fontsize, name=None, after=True):
         def generate_table_styles():
             s=Style(name="Table.Size{}".format(sum(sizes)), family='table')
@@ -449,17 +463,17 @@ class ODT(ODF):
             s.addElement(TextProperties(attributes={'fontsize':"{}pt".format(fontsize),  'fontweight':"bold"}))
             s.addElement(ParagraphProperties(attributes={'textalign':'center', }))
             self.doc.styles.addElement(s)
-            
+
             s = Style(name="Table.Contents.Font{}".format(fontsize), family="paragraph")
             s.addElement(TextProperties(attributes={'fontsize':"{}pt".format(fontsize), }))
             s.addElement(ParagraphProperties(attributes={'textalign':'justify', }))
             self.doc.styles.addElement(s)
-            
+
             s = Style(name="Table.ContentsRight.Font{}".format(fontsize), family="paragraph")
             s.addElement(TextProperties(attributes={'fontsize':"{}pt".format(fontsize), }))
             s.addElement(ParagraphProperties(attributes={'textalign':'end', }))
             self.doc.styles.addElement(s)
-            
+
             s = Style(name="Table.ContentsRight.FontRed{}".format(fontsize), family="paragraph")
             s.addElement(TextProperties(attributes={'fontsize':"{}pt".format(fontsize), 'color':'#ff0000' }))
             s.addElement(ParagraphProperties(attributes={'textalign':'end', }))
@@ -471,7 +485,6 @@ class ODT(ODF):
         ## @return TableCell
         def addTableCell(o, fontsize):
             tc = TableCell(stylename="Table.Cell")
-            
             #Parses orientation
             p = P(stylename="Table.ContentsRight.Font{}".format(fontsize))
             s=Span(text=o)
@@ -516,8 +529,22 @@ class ODT(ODF):
                 tr.addElement(addTableCell(o, fontsize))
         return self.insertInCursor(table, after)
 
-
-
+    ## Adds a paragraph with illustration style, with a list with image keys. All images will have the same width and height
+    ## @param image_key_list List with image keys to add to the paragraph
+    ## @param width Integer or float with number of width centimeters of the image in the document
+    ## @param height Integer or float with number of height centimeters of the image in the document
+    ## @param name String With the root name of the images in the paragraph. The root will be appended with a sequential number
+    ## @param after True: insert after self.cursor element. False: insert before self.cursor element. None: Just return element
+    def illustration(self, image_key_list, width, height, name, after=True):
+        p = P(stylename="Illustration")
+        for i, image_key in enumerate(image_key_list):
+            if name==None:
+                self.seqFrames=self.seqFrames+1
+                n="Frame.{}".format(self.seqFrames)
+            else:
+                n="{}.{}".format(name, i)
+            p.addElement(self.image(image_key, width, height, n))
+        self.insertInCursor(p, after=True)
 
 ## Class with starndard.odt template
 class ODT_Standard(ODT):
@@ -531,30 +558,6 @@ class ODT_Standard(ODT):
     def header(self, text, level, after=True):
         h=H(outlinelevel=level, stylename="Heading_20_{}".format(level), text=text)
         return self.insertInCursor(h, after)
-        
-#    ## Text body has no space between paragraph
-#    def list(self, arr, style='Standard', after=True):
-#        def item(o, style):
-#            it=ListItem()
-#            for obj in o:
-#                if obj==str:
-#                    p=P(stylename=style, text=item)
-#                    it.addElement(p)
-#                elif obj==List:
-#                    ls=List(stylename="List_20_1")
-#                    ls.addElement(item(obj, style))
-#                    it.addElement(ls)
-#            return it
-#        # #########################
-#        
-#        print("LIST MAL IN STANDARD")
-#        l=List(stylename="List_20_1", )
-#        for o in arr:
-#            l.addElement(item(o, style))
-#        self.insertInCursor(l, after)
-#        return l
-
-
 
     ## PH and PV styles are deffined in standard.odtg
     def pageBreak(self,  horizontal=False, after=True):    
@@ -563,6 +566,7 @@ class ODT_Standard(ODT):
         else:
             p=P(stylename="PV")
         return self.insertInCursor(p, after)
+
 ## Class with starndard.odt template
 class ODT_Manual_Styles(ODT):
     def __init__(self, filename, language="es", country="es"):
@@ -759,6 +763,8 @@ class OdfCell:
         return "OdfCell <{}{}>".format(self.coord.letter, self.coord.number)
 
     def generate(self):
+        if self.object==None:
+            self.object=" - "
         if self.object.__class__==Currency:
             odfcell = TableCell(valuetype="currency", currency=self.object.currency, value=self.object.amount, stylename=self.style)
         elif self.object.__class__==Percentage:
@@ -770,8 +776,12 @@ class OdfCell:
         elif self.object.__class__ in (Decimal, float,  int):
             odfcell= TableCell(valuetype="float", value=self.object,  stylename=self.style)
         else:#strings
-            if self.object[:1]=="=":#Formula
-                odfcell = TableCell(formula="of:"+self.object,  stylename=self.style)
+            if len(self.object)>0:
+                if self.object[:1]=="=":#Formula
+                    odfcell = TableCell(formula="of:"+self.object,  stylename=self.style)
+                else:
+                    odfcell = TableCell(valuetype="string", value=self.object,  stylename=self.style)
+                    odfcell.addElement(P(text = self.object))
             else:#Cadena
                 odfcell = TableCell(valuetype="string", value=self.object,  stylename=self.style)
                 odfcell.addElement(P(text = self.object))
@@ -905,20 +915,26 @@ B1:
                 return c
         return None
 
-    def add(self, coord, result, style):
+
+    ## Adds a cell to the sheet using its coord, an object and a color or a style
+    ## @param coord Coord where the cell is going to be created
+    ## @param result Object to add to the Cell. Can be int, str, float, datetime.datetime, datetime.date, Currency, Percentage, Decimal, None (will be converted to " - ")
+    ## @param color_or_style String with a color: Normal, White, Yellow, Orange, Blue, Red, GrayLight, GrayDark. Or a style WhiteInteger, YellowLeft, OrangeCenter, OrangeEUR, RedPercentage...
+    def add(self, coord, result, color_or_style="Normal"):
         coord=Coord.assertCoord(coord)
 
-        if result.__class__ in (str, int, float, datetime.datetime, datetime.date,  Currency, Percentage, Decimal):#Un solo valor
-            self.addCell(OdfCell(coord, result, style))
-        elif result.__class__ in (list,):#Una lista
+        if result.__class__ in (list,):#Una lista
             for i,row in enumerate(result):
-                if row.__class__ in (int, str, float, datetime.datetime,  datetime.date):#Una lista de una columna
-                    self.addCell(OdfCell(coord.addRow(i), result[i], style))
-                elif row.__class__ in (list, ):#Una lista de varias columnas
+                if row.__class__ in (list, ):#Una lista de varias columnas
                     for j, column in enumerate(row):
-                        self.addCell(OdfCell(coord.addColumn(j).addRow(i), result[i][j], style))
-                else:
-                    logging.warning(_("{} Class not found in '{}'").format(row.__class__, row))
+                        style=guess_ods_style(color_or_style, result[i][j])
+                        self.addCell(OdfCell(Coord(coord.string()).addColumn(j).addRow(i), result[i][j], style))
+                else: #Any value not list if row.__class__ in (int, str, float, datetime.datetime,  datetime.date, Currency, Percentage,  Decimal):#Una lista de una columna
+                    style=guess_ods_style(color_or_style, result[i])
+                    self.addCell(OdfCell(Coord(coord.string()).addRow(i), result[i], style))
+        else: #Any value not list#result.__class__ in (str, int, float, datetime.datetime, datetime.date,  Currency, Percentage, Decimal):#Un solo valor
+            style=guess_ods_style(color_or_style, result)  
+            self.addCell(OdfCell(coord, result, style))
 
     ## Adds a cell to self.arr with merge, content and style information
     ## @param range Range
@@ -1068,8 +1084,7 @@ class ODS(ODF):
     def setActiveSheet(self, value):
         """value is OdfSheet"""
         self.activeSheet=value.title
-        
-        
+
 class ODSStyleCurrency:
     def __init__(self, name, symbol):
         self.name=name
@@ -1277,19 +1292,26 @@ class ODS_Write(ODS):
         
 
 ## Guess style from color and object class
-def guess_ods_style(color, object):
-    if object.__class__==str:
-        return color + "Left"
-    if object.__class__==int:
-        return color + "Integer"
-    if object.__class__==Currency:
-        return color + "Euro"
-    if object.__class__==Percentage:
-        return color + "Percentage"
-    if object.__class__in (Decimal, float):
-        return color +  "Decimal2"
-    if object.__class__==datetime.datetime:
-        return color + "Datetime"
-    if object.__class__==datetime.date:
-        return color + "Date"
-    print("guess_ods_style not guessed",  object.__class__)
+## @param color_or_style String with a Color name or a style. If it's a color returns the style corresponding to the object. If it's a style returns the same style
+## @return String with the style 
+def guess_ods_style(color_or_style, object):
+    if color_or_style in ["Green", "GrayDark", "GrayLight", "Orange", "Yellow", "White", "Blue", "Red", "Normal"]:
+        if object.__class__==str:
+            return color_or_style + "Left"
+        elif object.__class__==int:
+            return color_or_style + "Integer"
+        elif object.__class__==Currency:
+            return color_or_style + object.currency
+        elif object.__class__==Percentage:
+            return color_or_style + "Percentage"
+        elif object.__class__ in (Decimal, float):
+            return color_or_style +  "Decimal2"
+        elif object.__class__==datetime.datetime:
+            return color_or_style + "Datetime"
+        elif object.__class__==datetime.date:
+            return color_or_style + "Date"
+        else:
+            logging.info("guess_ods_style not guessed",  object.__class__)
+            return "NormalLeft"
+    else:
+        return color_or_style
